# SPDX-FileCopyrightText: 2024-present Benjamin Piogé <benjamin@isahit.com>
#
# SPDX-License-Identifier: MIT

from .lab_iat_schema import LAB_IAT_SCHEMA
from .lab_iat_rectangle_schema import LAB_IAT_RECTANGLE_SCHEMA
from .lab_iat_segmentation_schema import LAB_IAT_SEGMENTATION_SCHEMA
from .lab_iat_graph_schema import LAB_IAT_GRAPH_SCHEMA
from .kili_schema import KILI_SCHEMA


__all__ = ['LAB_IAT_SCHEMA', 'LAB_IAT_RECTANGLE_SCHEMA', 'LAB_IAT_SEGMENTATION_SCHEMA', 'KILI_SCHEMA', 'LAB_IAT_GRAPH_SCHEMA']